<?php

namespace Drupal\Tests\book_moderation_sync\Functional;

use Drupal\Tests\BrowserTestBase;
use Drupal\node\Entity\Node;
use Drupal\book\Entity\Book;

/**
 * Tests the Book Moderation Sync functionality.
 *
 * @group book_moderation_sync
 */
class BookModerationSyncTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'book',
    'content_moderation',
    'book_moderation_sync',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with administrative permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    // Create an admin user.
    $this->adminUser = $this->drupalCreateUser([
      'administer nodes',
      'administer book outlines',
      'use content moderation',
    ]);
    $this->drupalLogin($this->adminUser);

    // Enable moderation for the 'book' content type.
    $this->container->get('entity_type.manager')
      ->getStorage('node_type')
      ->load('book')
      ->setThirdPartySetting('content_moderation', 'enabled', TRUE)
      ->save();
  }

  /**
   * Tests synchronization of moderation states between book and children.
   */
  public function testModerationStateSync() {
    // Create a book node.
    $book = Node::create([
      'type' => 'book',
      'title' => 'Test Book',
      'moderation_state' => 'draft',
    ]);
    $book->save();

    // Create a child node.
    $child = Node::create([
      'type' => 'book',
      'title' => 'Test Child',
      'moderation_state' => 'draft',
      'book' => [
        'bid' => $book->id(),
        'pid' => $book->id(),
      ],
    ]);
    $child->save();

    // Update the moderation state of the book node.
    $book->set('moderation_state', 'published');
    $book->save();

    // Reload the child node and check its moderation state.
    $child = Node::load($child->id());
    $this->assertEquals('published', $child->get('moderation_state')->value, 'Child node moderation state was synchronized.');
  }

  /**
   * Tests deletion of child nodes when a book is deleted.
   */
  public function testDeleteBookChildren() {
    // Create a book node.
    $book = Node::create([
      'type' => 'book',
      'title' => 'Test Book',
      'moderation_state' => 'draft',
    ]);
    $book->save();

    // Create a child node.
    $child = Node::create([
      'type' => 'book',
      'title' => 'Test Child',
      'moderation_state' => 'draft',
      'book' => [
        'bid' => $book->id(),
        'pid' => $book->id(),
      ],
    ]);
    $child->save();

    // Enable the "Delete children when a book is deleted" option.
    $this->config('book_moderation_sync.settings')
      ->set('delete_children_on_book_delete', TRUE)
      ->save();

    // Delete the book node.
    $book->delete();

    // Ensure the child node is deleted.
    $this->assertNull(Node::load($child->id()), 'Child node was deleted when the book was deleted.');
  }
}
